@@ -17,6 +17,8 @@ struct RarLocalTime
 
 class RarTime
 {
+  public:
+    static const uint TICKS_PER_SECOND = 10000000; // Raw time items per second.
   private:
     // Internal FILETIME like time representation in 100 nanoseconds
     // since 01.01.1601.
@@ -39,11 +41,12 @@ class RarTime
     bool operator >= (RarTime &rt) {return itime>rt.itime || itime==rt.itime;}
     void GetLocal(RarLocalTime *lt);
     void SetLocal(RarLocalTime *lt);
+    void SetUTC(RarLocalTime *lt);
     uint64 GetRaw();
     void SetRaw(uint64 RawTime);
     uint GetDos();
     void SetDos(uint DosTime);
-    void GetText(wchar *DateStr,size_t MaxSize,bool FullYear,bool FullMS);
+    void GetText(wchar *DateStr,size_t MaxSize,bool FullMS);
     void SetIsoText(const wchar *TimeText);
     void SetAgeText(const wchar *TimeText);
     void SetCurrentTime();
