@@ -43,6 +43,7 @@ void Unpack::InitMT()
       {
         // Typical number of items in RAR blocks does not exceed 0x4000.
         CurData->DecodedAllocated=0x4100;
+        // It will be freed in the object destructor, not in this file.
         CurData->Decoded=(UnpackDecodedItem *)malloc(CurData->DecodedAllocated*sizeof(UnpackDecodedItem));
         if (CurData->Decoded==NULL)
           ErrHandler.MemoryError();
@@ -90,10 +91,12 @@ void Unpack::Unpack5MT(bool Solid)
     DataSize+=ReadSize;
     if (DataSize==0)
       break;
+
+    // First read chunk can be small if we are near the end of volume
+    // and we want it to fit block header and tables.
     if (ReadSize>0 && DataSize<TooSmallToProcess)
       continue;
 
-    bool BufferProcessed=false;
     while (BlockStart<DataSize && !Done)
     {
       uint BlockNumber=0,BlockNumberMT=0;
@@ -272,6 +275,7 @@ void Unpack::Unpack5MT(bool Solid)
       }
     }
   }
+  UnpPtr&=MaxWinMask; // ProcessDecoded and maybe others can leave UnpPtr > MaxWinMask here.
   UnpWriteBuf();
 
   BlockHeader=UnpThreadData[LastBlockNum].BlockHeader;
@@ -325,9 +329,10 @@ void Unpack::UnpackDecode(UnpackThreadData &D)
     if (D.DecodedSize>D.DecodedAllocated-8) // Filter can use several slots.
     {
       D.DecodedAllocated=D.DecodedAllocated*2;
-      D.Decoded=(UnpackDecodedItem *)realloc(D.Decoded,D.DecodedAllocated*sizeof(UnpackDecodedItem));
-      if (D.Decoded==NULL)
-        ErrHandler.MemoryError();
+      void *Decoded=realloc(D.Decoded,D.DecodedAllocated*sizeof(UnpackDecodedItem));
+      if (Decoded==NULL)
+        ErrHandler.MemoryError(); // D.Decoded will be freed in the destructor.
+      D.Decoded=(UnpackDecodedItem *)Decoded;
     }
 
     UnpackDecodedItem *CurItem=D.Decoded+D.DecodedSize++;
